module Kojak
  # Internal: Registers investigator within given class for specified method.
  #
  # cls - The Class that will be affected.
  # m   - The Method to override with investigation support.
  #
  # Returns nothing.
  def self.register_investigator_for!(cls, m)
    cls.send(:define_method, m.name) do |*args,&block|
      begin
        caller = Caller.new(m.bind(self))
        caller.call(*args,&block)
      ensure
        Kojak.print(caller) if caller
      end
    end
  end

  # Public: Extend your class with this inspector module to get
  # investigation functionality.
  #
  # Example
  #
  #   class Dummy
  #     investigate :foo, :bar
  #
  #     def foo
  #       *snip*...
  #     end
  #
  #     def bar
  #       *snip*...
  #     end
  #   end
  #
  #   Dummy.investigate! # ... or Kojak.investigate_all!
  #
  module Inspector
    def self.extended(cls)
      Kojak.mark_for_investigation(cls)
    end

    # Public: Marks given methods for investigation.
    #
    # names - The Array of method names to investigate.
    #
    # Returns Array with names of all methods being under investigation.
    def investigate(*names)
      Kojak.mark_for_investigation(self)
      @__kojak_investigate ||= []
      @__kojak_investigate.concat(names).uniq!
    end

    # Public: Enables investigation of caller class.
    def investigate!
      @__kojak_investigate.each do |name|
        m = instance_method(name)
        Kojak.register_investigator_for!(self, m)
      end
    end
  end
end
